@@ -52,19 +52,57 @@ def qname(func):
     m = func.__module__
     return m and m+'.'+func.__name__ or func.__name__
 
+class Bomb:
+    def __str__(self):
+        raise RuntimeError("template functions must return a static string!")
+bomb = Bomb()
+
+def getbody(func):
+    from inspect import getargspec
+    args, varargs, kwargs, defaults = getargspec(func)
+    return func(*[bomb] * len(args))
+    
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 def apply_template(wrapper, func, *args, **kw):
+
     funcname, argspec  = name_and_spec(func)
     wrapname, wrapspec = name_and_spec(wrapper)
-    body = wrapper.__doc__.replace('%','%%').replace('$args','%(argspec)s')
-    d ={}
+
+    body = wrapper.__doc__ or getbody(wrapper)
+    if not body:
+        raise RuntimeError(
+            "Missing docstring or empty return value from"
+            " %s(%s) - please switch the calling code from using docstrings"
+            " to return values" % (wrapname, wrapspec)
+        )
+
+    body = body.replace('%','%%').replace('$args','%(argspec)s')
     body = """
 def %(wrapname)s(%(wrapspec)s):
  def %(funcname)s(%(argspec)s): """+body+"""
  return %(funcname)s
 """
     body %= locals()
+    
     filename = "<%s wrapping %s at 0x%08X>" % (qname(wrapper), qname(func), id(func))
+    d ={}
     exec compile(body, filename, "exec") in func.func_globals, d
 
     f = d[wrapname](func, *args, **kw)
@@ -80,6 +118,9 @@ def %(wrapname)s(%(wrapspec)s):
 
 
 
+
+
+
 def rewrap(func, wrapper):
     """Create a wrapper with the signature of `func` and a body of `wrapper`
 
@@ -110,7 +151,7 @@ def rewrap(func, wrapper):
     to ``rewrap()``.
     """
     def rewrap(__original, __decorated):
-        """return __decorated($args)"""
+        return """return __decorated($args)"""
     return apply_template(rewrap, func, wrapper)
 
 
@@ -301,8 +342,11 @@ def synchronized(func=None):
     if func is None:
         return decorate_assignment(lambda f,k,v,o: synchronized(v))
 
+    from inspect import getargspec
+    first_arg = getargspec(func)[0][0]
+
     def wrap(__func):
-        '''
+        return '''
         try:
             lock = $self.__lock__
         except AttributeError:
@@ -315,12 +359,9 @@ def synchronized(func=None):
         try:
             return __func($args)
         finally:
-            lock.release()'''
-    from inspect import getargspec
-    first_arg = getargspec(func)[0][0]
-    wrap.__doc__ = wrap.__doc__.replace('$self', first_arg)
-    return apply_template(wrap, func)
+            lock.release()'''.replace('$self', first_arg)
 
+    return apply_template(wrap, func)
 
 
 
@@ -618,7 +659,13 @@ def super_next(cls, attr):
         if attr in c.__dict__:
             yield getattr(c, attr).im_func
 
-class classy_class(type):
+# Python 2.6 and above mix ABCMeta into various random places  :-(
+try:
+    from abc import ABCMeta as base
+except ImportError:
+    base = type
+
+class classy_class(base):
     """Metaclass that delegates selected operations back to the class"""
 
     def __new__(meta, name, bases, cdict):
@@ -633,6 +680,21 @@ class classy_class(type):
     def __call__(cls, *args, **kw):
         return cls.__class_call__.im_func(cls, *args, **kw)
 
+    if base is not type:
+        # Our instances do not support ABC-ness
+        def register(*args): raise NotImplementedError
+        __instancecheck__ = type.__dict__['__instancecheck__']
+        __subclasscheck__ = type.__dict__['__subclasscheck__']
+
+
+
+
+
+
+
+
+
+
 
 class classy(object):
     """Base class for classes that want to be their own metaclass"""
@@ -654,3 +716,24 @@ class classy(object):
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
